;---------------------------------------
;
;
;  BURST COMMAND
;  22/5/85
;  BY S.TOKUNAGA
;---------------------------------------
;
SETLFS = $FFBA ; SET LOGICAL FILE #,DEVICE #,CHANNEL#
SETNAM = $FFBD ;
OPEN = $FFC0
CHKOUT = $FFC9
CLRCHN = $FFCC
CHROUT = $FFD2 ; OUTPUT BYTE TO CURRENT OUTPUT REG
;
SEDATA = $DC0C ; SERIAL DATA REGISTER
ICR = $DC0D ; INTERRUPT CONTROL REGISTER
CRA = $DC0E ; CONTROL REG A
;
SEPORT = $DD00 ; CIA#2 PERIPHERAL DATA REG
;
STATUS = $FA
BUFF = $FB ; READ/WRITE BUFFER ADDRESS
CMDP = $FD ; COMMAND POINTER ($FD),($FE)
;
;----------------------------------------
;  JUMP TABL
;   INPUT : SIDE =0 ,$10
;           TRACK=0 - 40
;           SECTR=0 - 26
;           SCSIZE=0,1,2,4
;----------------------------------------
* = $1800
 JMP READ
 JMP WRITE
 JMP INQUI
 JMP QUERY
;
 JMP CMPR ; DATA COMPARE
 JMP FILL ; FILL DATA
;
;----------------------------------------
;  COMMAND TABL
;----------------------------------------
CMD3 .BYTE 3,'U0',%00000100,0,0,0,0 ;INQUIRE
;
CMD6 .BYTE 4,'U0',%00001010,2,0,0,0 ;QUERY
CMD1 .BYTE 6,'U0',%00000000,2,1,1,0 ;READ
CMD2 .BYTE 7,'U0',%00000010,2,1,1,0 ;WRITE
;----------------------------------------
;  WORK AREA
;----------------------------------------
STSQ * =*+7 ; STATUS STORE AREA
;
SIDE .BYTE 0
TRACK .BYTE 2
SECTR .BYTE 1
SCSIZE .BYTE 1 ; SECTOR SIZE 0,1,2,4
;                128=0,256=1,512=2,1024=4
CSTS .BYTE 0 ; COMPARE RESULT STATUS
WDATA .BYTE 0 ; WRITE DATA
;----------------------------------------
;  COMMAND FORMAT
;----------------------------------------
;  INQUIRE
;  'U0',%00000100
;        IIII I I
;        IIII I I
;        IIII I +--- DRIVE #0
;        IIII +----- INQUIRE STATUS
;        III+------- SIDE 0 OR 1
;        II+-------- X
;        I+--------- X
;        +---------- X
;
;  QUERY DISK FORMAT
;  'U0',%00001010,TRACK
;        IIII I I
;        IIII I +--- DRIVE #0
;        IIII +----- QUERY
;        III+------- SIDE 0 OR 1
;        II+-------- X
;        I+--------- X
;        +---------- X
;
;  READ
;  'U0',%TEBS000N,TRACK,SECTOR,NSECTOR,NTRACK
;        IIII---I  I     I      I       I
;        IIII I I  I     I      I       +- OPTIONAL
;        IIII I I  I     I      +--- # OF SECTORS TO BE READ
;        IIII I I  I     +--- DESTINATION SECTOR
;        IIII I I  +--- DESTINATION TRACK
;        IIII I I
;        IIII I +--- DRIVE# 0 OR 1
;        IIII +----- BURST READ
;        III+------- SIDE SELECT (MFM ONLY)
;        II+-------- BUFFER TRANSFER ONLY =1
;        I+--------- IGNORE ERROR (1=IGNORE)
;        +---------- TRANSFER DATA (1=NO TRANSFER)
;
;  WRITE
;  'U0',%TEBS000N,TRACK,SECTOR,NSECTOR,NTRACK
;        IIII---I  I     I      I       I
;        IIII I I  I     I      I       +- OPTIONAL
;        IIII I I  I     I      +--- # OF SECTORS TO BE WRITE
;        IIII I I  I     +--- DESTINATION SECTOR
;        IIII I I  +--- DESTINATION TRACK
;        IIII I I
;        IIII I +--- DRIVE# 0 OR 1
;        IIII +----- BURST WRITE
;        III+------- SIDE SELECT (MFM ONLY)
;        II+-------- BUFFER TRANSFER ONLY =1
;        I+--------- IGNORE ERROR (1=IGNORE)
;        +---------- TRANSFER DATA (1=NO TRANSFER)
;
;----------------------------------------
; SUBROUTINE COMMAND OUTPUT
; COMMAND CHANNEL MUST BE OPEN ON DRIVE
; OPEN 15,8,15
;  INPUT : (CMDP)= COMMAND ADDRESS
;----------------------------------------
CMDOUT
 LDY #$3 ; POINTER
 LDA SIDE ;(SIDE)=0 OR 1
 BEQ CMDO0 ; 0
 LDA #$10 ; 1
 ORA (CMDP),Y ; SIDE BIT =1
 JMP CMDO1
CMDO0 LDA #$EF
 AND (CMDP),Y ; SIDE BIT =0
CMDO1 STA (CMDP),Y
;
 LDA #$0F ; LOGICAL FILE NUMBER
 LDX #$08 ; DEVICE NUMBER
 LDY #$0F ; SECONDARY ADDRESS
 JSR SETLFS ; SETUP LOGICAL FILE
 LDA #0 ; NONAME
 JSR SETNAM ; SETUP FILENAME
 JSR OPEN ; OPEN LOGICAL FILE
 LDX #15 ;
 JSR CHKOUT ; OPEN CHANNEL FOR OUTPUT
 LDY #0
 LDA (CMDP),Y ; BYTE COUNT LOAD
 TAX ; X = BYTE COUNT
 INY ; Y = 1 ,POINTER
SENDIT LDA (CMDP),Y ; GET  COMMAND
 JSR CHROUT ; SEND COMMAND
 INY ; POINTER +1
 DEX ; COUNTER -1
 BNE SENDIT ; NEXT BYTE
;
 JSR CLRCHN ; SEND EOI
;
 RTS
;
;
;----------------------------------------
;  CMD 3 : INQUIRE DISK TYPE
;----------------------------------------
INQUI
 LDA #<CMD3
 STA CMDP
 LDA #>CMD3
 STA CMDP+1 ; COMMAND ADDRESS SET
 JSR CMDOUT ; COMMAND OUTPUT
;
 SEI
 LDY #$00
 BIT ICR
 JSR PRT
WAIT1 LDA ICR
 AND #$08
 BEQ WAIT1
 LDA SEDATA ; QET DATA FROM SERIAL DATA REG
 STA STATUS ; STORE DATA
 CLC
 CLI
 RTS
;
PRT LDA SEPORT
 EOR #$10 ; %0001 0000
 STA SEPORT
 RTS
;
;
;
;
;
;
;----------------------------------------
;  CMD 6 : QUERY DISK FORMAT
;----------------------------------------
;
QUERY
 LDA TRACK ; TRACK SET
 STA CMD6+4
;
 LDA #<CMD6 ; COMMAND ADDRESS SET
 STA CMDP
 LDA #>CMD6
 STA CMDP+1
;
 JSR CMDOUT ; COMMAND OUT PUT
;
;
 SEI
 LDY #$00
  BIT ICR
QUERY1 JSR PRT
 JSR WAIT
 LDA SEDATA ; GET STATUS
 STA STSQ ; STORE STATUS
 AND #$0F
 CMP #$02 ; ERROR ?
 BCS QUEER
 BIT STSQ   ; BIT 7 SET ?
 BPL QUEER
;
QUERY2 JSR PRT
 JSR WAIT
 LDA SEDATA ; GET STATUS
 STA STSQ+1
 AND #$0F
 CMP #$02 ; ERROR ?
 BCS QUEER
;
QUERY3 JSR PRT
 JSR WAIT
 LDA SEDATA
 STA STSQ+2,Y
 INY
 CPY #$05
 BNE QUERY3
QUEER CLC
 CLI
 RTS
;
WAIT LDA ICR
 AND #$08
 BEQ WAIT
 RTS
;
;
;
;
;
;----------------------------------------
;  CMD 1 : READ
;  INPUT : TRACK,SECTOR,SECTOR SIZE
;          BUFFER ADDRESS
;----------------------------------------
OLDCLK = $0A1C ; FAST SERIAL FLAG
;
CKINH = $BF ; %1011 1111    BIT6
CKINL = $40 ; %0100 0000    BIT6
CKOUTH = $EF ; %1110 1111    BIT4
CKOUTL = $10 ; %0001 0000    BIT4
;
;
READ
 LDA TRACK ; TRACK SET
 STA CMD1+4
 LDA SECTR ; SECTER SET
 STA CMD1+5
;
 LDA #<CMD1
 STA CMDP
 LDA #>CMD1
 STA CMDP+1 ; COMMAND ADDRESS SET
;
 JSR CMDOUT ; COMMAND OUTPUT
;
 JSR READST
 BIT ICR ; CHECK INTERRUPT CONTROL REG
;
READIT LDY #$00 ; POINTER 0 SET
 LDX SCSIZE
 BNE READ0
 LDY #$80 ; 128 BYTE
 INX ; X=1 SET
READ0 JSR TOGGLE
 JSR WAIT ; WAIT FOR BYTE
;
 LDA SEDATA ; GET DATA
 STA (BUFF),Y ; STORE BUFFER
;
 INY
 BNE READ0
;
 INC BUFF+1
 DEX
 BNE READ0
;
;
 JSR CLKHI ; CLOCK HIGH
;
 CLI
 CLC
 RTS
;
; SUBROUTINES
;
TOGGLE LDA SEPORT
 EOR #CKOUTL
 STA SEPORT
 RTS
;
CLKHI LDA SEPORT
 AND #CKOUTH
 STA SEPORT
 RTS
;
READST SEI
 LDY #$00
 BIT ICR
 JSR TOGGLE
 JSR WAIT
 LDA SEDATA ; GET DATA
 STA STATUS
 AND #$0F
 CMP #$02 ; ERROR ?
 BCS REDER
 RTS ; NORMAL
REDER CLI
 CLC
 RTS
;
;
;
;
;----------------------------------------
;  CMD 2 : WRITE
;  INPUT : TRACK,SECTER,SECTOR SIZE, SIDE
;          BUFFER DATA
;----------------------------------------
MMUMCR = $D505 ; MMU REGISTER
;
TALO = $DC04 ; TIMER A LOW REG
TAHI = $DC05 ; TIMER A HIGH REG
;
WRITE
 LDA TRACK ; TRACK SET
 STA CMD2+4
 LDA SECTR ; SECTER SET
 STA CMD2+5
;
 LDA #<CMD2
 STA CMDP
 LDA #>CMD2
 STA CMDP+1
;
 JSR CMDOUT ; COMMAND OUTPUT
;
 BIT ICR ; CHECK SPEED OF DRIVE
 LDA OLDCLK ; CHECK STATUS
 ORA #CKINL ; CLEAR BIT 6 FAST SERIAL FLAG
 STA OLDCLK
;
 SEI
WRIT0 JSR SPOUT ; SERIAL PORT OUTPUT
 LDY #$00
 LDX SCSIZE ; SECTOR SIZE
 BNE WR1
 LDY #$80 ; 128 BYTES
 INX ; X=1 SET
;
WR1 LDA SEPORT ; CHECK CLOCK
 CMP SEPORT
 BNE WR1
;
 EOR OLDCLK
 AND #CKINL
 BEQ WR1
;
 LDA OLDCLK ; CHANGE STATUS OF OLD CLOCK
 EOR #CKINL
 STA OLDCLK
;
 LDA (BUFF),Y ; SEND DATA
 STA SEDATA
 LDA #$00
 ;
 JSR WWAIT ; WAIT FOR TRANS TIME
;
 INY
 BNE WR1
;
 INC BUFF+1
 DEX
 BNE WR1
;
 JSR RDST1 ; READ BURST STATUS
;
 JSR CLKHI
 CLC
 CLI
 RTS
;
;
; GET BURST STATUS ROUTINE
;
RDST1 JSR SPINP ; SERIAL PORT INPUT
 SEI
 BIT ICR
 JSR CLKLO
;
 JSR WWAIT ; WAIT FOR STATUS BYTE
;
 LDA SEDATA ;
 STA STATUS
 AND #$0F
 CMP #$02
 BCS RDST10
 JSR SPOUT
 JSR CLKHI
 RTS
RDST10 JSR SPOUT
 JSR CLKHI
 CLI
 CLC
 RTS
;
;
SPOUT LDA MMUMCR
 ORA #$08
 STA MMUMCR
 LDA #$7F
 STA ICR
 LDA #$00
 STA TAHI
 LDA #$04
 STA TALO
 LDA CRA
 AND #$80
 ORA #$55
 STA CRA
 BIT ICR
 RTS
;
SPINP LDA #$08
 STA CRA
 LDA MMUMCR
 AND #$F7
 STA MMUMCR
 RTS
;
CLKLO LDA SEPORT
 ORA #CKOUTL
 STA SEPORT
 RTS
;
WWAIT LDA #$08
WWAIT1 BIT ICR
 BEQ WWAIT1
 RTS
;
;
;
;
;
;
;
;----------------------------------------
;  DATA COMPARE SUBROUTINE
;  READ DATA = 2000H - 23FFH
;  WRITE DATA= 2400H - 27FFH
;  INPUT : SSIZE=0,1,2,4
;  OUTPUT: CSTS = 0 : O.K, =1 : ERROR
;----------------------------------------
P0 = $FB ; READ DATA POINTER
P1 = $FD ; WRITE DATA POINTER
;
CMPR
 LDA #$00
 STA P0
 STA P1
 LDA #$20
 STA P0+1
 LDA #$24
 STA P1+1 ; DATA POINTER SET
;
 LDY #$00 ; SECTER SIZE
 LDX SCSIZE
 BNE CMPR0
 LDY #$80 ; 128 BYTES
 INX ; X=1 SET
CMPR0
 LDA (P0),Y ; READ DATA LOAD
 CMP (P1),Y ; COMPARE
 BNE CMPER ; ERROR
 INY ; POINTER +1
 BNE CMPR0
;
 INC P0+1
 INC P1+1
 DEX ; SECTER SIZE -1
 BNE CMPR0
;
 LDA #$00
 STA CSTS
 RTS
;
CMPER
 LDA #$01
 STA CSTS ; ERROR STS
 RTS
;
;
;
;----------------------------------------
;   FILL DATA SUBROUTINE
;   2400H - 27FFH
;   INPUT : (WDATA)= FILL DATA
;----------------------------------------
FILL
 LDA #$0
 STA P0
 LDA #$24
 STA P0+1 ; ADDRESS SET
;
 LDX #$4 ; 256 COUNTER
 LDY #$0 ; POINTER=0 SET
 LDA WDATA ; FILL DATA LOAD
FILL0
 STA (P0),Y ; DATA WRITE
 INY
 BNE FILL0
 INC P0+1
 DEX
 BNE FILL0
 RTS
;
;
 .END
